@@ -33,18 +33,15 @@
 #include <alloca.h>
 #endif
 
-#include <glib.h>
 #include <glib/gstdio.h>
 #include <glib/gi18n-lib.h>
 
 #include "camel-data-cache.h"
-#include "camel-exception.h"
+#include "camel-object-bag.h"
 #include "camel-stream-fs.h"
 #include "camel-stream-mem.h"
 #include "camel-file-utils.h"
 
-extern gint camel_verbose_debug;
-#define dd(x) (camel_verbose_debug?(x):0)
 #define d(x)
 
 /* how many 'bits' of hash are used to key the toplevel directory */
@@ -55,69 +52,119 @@ extern gint camel_verbose_debug;
    once an hour should be enough */
 #define CAMEL_DATA_CACHE_CYCLE_TIME (60*60)
 
+#define CAMEL_DATA_CACHE_GET_PRIVATE(obj) \
+	(G_TYPE_INSTANCE_GET_PRIVATE \
+	((obj), CAMEL_TYPE_DATA_CACHE, CamelDataCachePrivate))
+
 struct _CamelDataCachePrivate {
 	CamelObjectBag *busy_bag;
 
+	gchar *path;
+
+	time_t expire_age;
+	time_t expire_access;
+
 	time_t expire_last[1<<CAMEL_DATA_CACHE_BITS];
 };
 
-static CamelObject *camel_data_cache_parent;
+enum {
+	PROP_0,
+	PROP_PATH
+};
 
-static void data_cache_class_init(CamelDataCacheClass *klass)
+G_DEFINE_TYPE (CamelDataCache, camel_data_cache, CAMEL_TYPE_OBJECT)
+
+static void
+data_cache_set_property (GObject *object,
+                         guint property_id,
+                         const GValue *value,
+                         GParamSpec *pspec)
 {
-	camel_data_cache_parent = (CamelObject *)camel_object_get_type ();
-
-#if 0
-	klass->add = data_cache_add;
-	klass->get = data_cache_get;
-	klass->close = data_cache_close;
-	klass->remove = data_cache_remove;
-	klass->clear = data_cache_clear;
-#endif
-}
-
-static void data_cache_init(CamelDataCache *cdc, CamelDataCacheClass *klass)
-{
-	struct _CamelDataCachePrivate *p;
-
-	p = cdc->priv = g_malloc0(sizeof(*cdc->priv));
-	p->busy_bag = camel_object_bag_new(g_str_hash, g_str_equal, (CamelCopyFunc)g_strdup, g_free);
-}
-
-static void data_cache_finalise(CamelDataCache *cdc)
-{
-	struct _CamelDataCachePrivate *p;
-
-	p = cdc->priv;
-	camel_object_bag_destroy(p->busy_bag);
-	g_free(p);
-
-	g_free (cdc->path);
-}
-
-CamelType
-camel_data_cache_get_type(void)
-{
-	static CamelType camel_data_cache_type = CAMEL_INVALID_TYPE;
-
-	if (camel_data_cache_type == CAMEL_INVALID_TYPE) {
-		camel_data_cache_type = camel_type_register(
-			CAMEL_OBJECT_TYPE, "CamelDataCache",
-			sizeof (CamelDataCache),
-			sizeof (CamelDataCacheClass),
-			(CamelObjectClassInitFunc) data_cache_class_init,
-			NULL,
-			(CamelObjectInitFunc) data_cache_init,
-			(CamelObjectFinalizeFunc) data_cache_finalise);
+	switch (property_id) {
+		case PROP_PATH:
+			camel_data_cache_set_path (
+				CAMEL_DATA_CACHE (object),
+				g_value_get_string (value));
+			return;
 	}
 
-	return camel_data_cache_type;
+	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+}
+
+static void
+data_cache_get_property (GObject *object,
+                         guint property_id,
+                         GValue *value,
+                         GParamSpec *pspec)
+{
+	switch (property_id) {
+		case PROP_PATH:
+			g_value_set_string (
+				value, camel_data_cache_get_path (
+				CAMEL_DATA_CACHE (object)));
+			return;
+	}
+
+	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+}
+
+static void
+data_cache_finalize (GObject *object)
+{
+	CamelDataCachePrivate *priv;
+
+	priv = CAMEL_DATA_CACHE_GET_PRIVATE (object);
+
+	camel_object_bag_destroy (priv->busy_bag);
+	g_free (priv->path);
+
+	/* Chain up to parent's finalize() method. */
+	G_OBJECT_CLASS (camel_data_cache_parent_class)->finalize (object);
+}
+
+static void
+camel_data_cache_class_init (CamelDataCacheClass *class)
+{
+	GObjectClass *object_class;
+
+	g_type_class_add_private (class, sizeof (CamelDataCachePrivate));
+
+	object_class = G_OBJECT_CLASS (class);
+	object_class->set_property = data_cache_set_property;
+	object_class->get_property = data_cache_get_property;
+	object_class->finalize = data_cache_finalize;
+
+	g_object_class_install_property (
+		object_class,
+		PROP_PATH,
+		g_param_spec_string (
+			"path",
+			"Path",
+			NULL,
+			NULL,
+			G_PARAM_READWRITE |
+			G_PARAM_CONSTRUCT));
+}
+
+static void
+camel_data_cache_init (CamelDataCache *data_cache)
+{
+	CamelObjectBag *busy_bag;
+
+	busy_bag = camel_object_bag_new (
+		g_str_hash, g_str_equal,
+		(CamelCopyFunc) g_strdup,
+		(GFreeFunc) g_free);
+
+	data_cache->priv = CAMEL_DATA_CACHE_GET_PRIVATE (data_cache);
+	data_cache->priv->busy_bag = busy_bag;
+	data_cache->priv->expire_age = -1;
+	data_cache->priv->expire_access = -1;
 }
 
 /**
  * camel_data_cache_new:
  * @path: Base path of cache, subdirectories will be created here.
- * @flags: Open flags, none defined.
  * @ex:
  *
  * Create a new data cache.
@@ -126,24 +173,57 @@ camel_data_cache_get_type(void)
  * be written to.
  **/
 CamelDataCache *
-camel_data_cache_new(const gchar *path, guint32 flags, CamelException *ex)
+camel_data_cache_new (const gchar *path,
+                      CamelException *ex)
 {
-	CamelDataCache *cdc;
+	g_return_val_if_fail (path != NULL, NULL);
 
 	if (g_mkdir_with_parents (path, 0700) == -1) {
-		camel_exception_setv(ex, CAMEL_EXCEPTION_SYSTEM,
-				     _("Unable to create cache path"));
+		camel_exception_setv (
+			ex, CAMEL_EXCEPTION_SYSTEM,
+			_("Unable to create cache path"));
 		return NULL;
 	}
 
-	cdc = (CamelDataCache *)camel_object_new(CAMEL_DATA_CACHE_TYPE);
+	return g_object_new (CAMEL_TYPE_DATA_CACHE, "path", path, NULL);
+}
 
-	cdc->path = g_strdup(path);
-	cdc->flags = flags;
-	cdc->expire_age = -1;
-	cdc->expire_access = -1;
+/**
+ * camel_data_cache_get_path:
+ * @cdc: a #CamelDataCache
+ *
+ * Returns the path to the data cache.
+ *
+ * Returns: the path to the data cache
+ *
+ * Since: 3.0
+ **/
+const gchar *
+camel_data_cache_get_path (CamelDataCache *cdc)
+{
+	g_return_val_if_fail (CAMEL_IS_DATA_CACHE (cdc), NULL);
 
-	return cdc;
+	return cdc->priv->path;
+}
+
+/**
+ * camel_data_cache_set_path:
+ * @cdc: a #CamelDataCache
+ * @path: path to the data cache
+ *
+ * Sets the path to the data cache.
+ *
+ * Since: 3.0
+ **/
+void
+camel_data_cache_set_path (CamelDataCache *cdc,
+                           const gchar *path)
+{
+	g_return_if_fail (CAMEL_IS_DATA_CACHE (cdc));
+	g_return_if_fail (path != NULL);
+
+	g_free (cdc->priv->path);
+	cdc->priv->path = g_strdup (path);
 }
 
 /**
@@ -164,7 +244,7 @@ camel_data_cache_new(const gchar *path, guint32 flags, CamelException *ex)
 void
 camel_data_cache_set_expire_age(CamelDataCache *cdc, time_t when)
 {
-	cdc->expire_age = when;
+	cdc->priv->expire_age = when;
 }
 
 /**
@@ -185,7 +265,7 @@ camel_data_cache_set_expire_age(CamelDataCache *cdc, time_t when)
 void
 camel_data_cache_set_expire_access(CamelDataCache *cdc, time_t when)
 {
-	cdc->expire_access = when;
+	cdc->priv->expire_access = when;
 }
 
 static void
@@ -207,17 +287,15 @@ data_cache_expire(CamelDataCache *cdc, const gchar *path, const gchar *keep, tim
 			continue;
 
 		g_string_printf (s, "%s/%s", path, dname);
-		dd(printf("Checking '%s' for expiry\n", s->str));
 		if (g_stat(s->str, &st) == 0
 		    && S_ISREG(st.st_mode)
-		    && ((cdc->expire_age != -1 && st.st_mtime + cdc->expire_age < now)
-			|| (cdc->expire_access != -1 && st.st_atime + cdc->expire_access < now))) {
-			dd(printf("Has expired!  Removing!\n"));
+		    && ((cdc->priv->expire_age != -1 && st.st_mtime + cdc->priv->expire_age < now)
+			|| (cdc->priv->expire_access != -1 && st.st_atime + cdc->priv->expire_access < now))) {
 			g_unlink(s->str);
 			stream = camel_object_bag_get(cdc->priv->busy_bag, s->str);
 			if (stream) {
 				camel_object_bag_remove(cdc->priv->busy_bag, stream);
-				camel_object_unref(stream);
+				g_object_unref (stream);
 			}
 		}
 	}
@@ -237,8 +315,8 @@ data_cache_path(CamelDataCache *cdc, gint create, const gchar *path, const gchar
 
 	hash = g_str_hash(key);
 	hash = (hash>>5)&CAMEL_DATA_CACHE_MASK;
-	dir = alloca(strlen(cdc->path) + strlen(path) + 8);
-	sprintf(dir, "%s/%s/%02x", cdc->path, path, hash);
+	dir = alloca(strlen(cdc->priv->path) + strlen(path) + 8);
+	sprintf(dir, "%s/%s/%02x", cdc->priv->path, path, hash);
 
 #ifdef G_OS_WIN32
 	if (g_access(dir, F_OK) == -1) {
@@ -247,11 +325,9 @@ data_cache_path(CamelDataCache *cdc, gint create, const gchar *path, const gchar
 #endif
 		if (create)
 			g_mkdir_with_parents (dir, 0700);
-	} else if (cdc->expire_age != -1 || cdc->expire_access != -1) {
+	} else if (cdc->priv->expire_age != -1 || cdc->priv->expire_access != -1) {
 		time_t now;
 
-		dd(printf("Checking expire cycle time on dir '%s'\n", dir));
-
 		/* This has a race, but at worst we re-run an expire cycle which is safe */
 		now = time(NULL);
 		if (cdc->priv->expire_last[hash] + CAMEL_DATA_CACHE_CYCLE_TIME < now) {
@@ -286,7 +362,10 @@ data_cache_path(CamelDataCache *cdc, gint create, const gchar *path, const gchar
  * The caller must unref this when finished.
  **/
 CamelStream *
-camel_data_cache_add(CamelDataCache *cdc, const gchar *path, const gchar *key, CamelException *ex)
+camel_data_cache_add (CamelDataCache *cdc,
+                      const gchar *path,
+                      const gchar *key,
+                      CamelException *ex)
 {
 	gchar *real;
 	CamelStream *stream;
@@ -300,11 +379,12 @@ camel_data_cache_add(CamelDataCache *cdc, const gchar *path, const gchar *key, C
 		if (stream) {
 			g_unlink(real);
 			camel_object_bag_remove(cdc->priv->busy_bag, stream);
-			camel_object_unref(stream);
+			g_object_unref (stream);
 		}
 	} while (stream != NULL);
 
-	stream = camel_stream_fs_new_with_name(real, O_RDWR|O_CREAT|O_TRUNC, 0600);
+	stream = camel_stream_fs_new_with_name (
+		real, O_RDWR|O_CREAT|O_TRUNC, 0600);
 	if (stream)
 		camel_object_bag_add(cdc->priv->busy_bag, real, stream);
 	else
@@ -330,7 +410,10 @@ camel_data_cache_add(CamelDataCache *cdc, const gchar *path, const gchar *key, C
  * Returns: A cache item, or NULL if the cache item does not exist.
  **/
 CamelStream *
-camel_data_cache_get(CamelDataCache *cdc, const gchar *path, const gchar *key, CamelException *ex)
+camel_data_cache_get (CamelDataCache *cdc,
+                      const gchar *path,
+                      const gchar *key,
+                      CamelException *ex)
 {
 	gchar *real;
 	CamelStream *stream;
@@ -338,7 +421,8 @@ camel_data_cache_get(CamelDataCache *cdc, const gchar *path, const gchar *key, C
 	real = data_cache_path(cdc, FALSE, path, key);
 	stream = camel_object_bag_reserve(cdc->priv->busy_bag, real);
 	if (!stream) {
-		stream = camel_stream_fs_new_with_name(real, O_RDWR, 0600);
+		stream = camel_stream_fs_new_with_name (
+			real, O_RDWR, 0600);
 		if (stream)
 			camel_object_bag_add(cdc->priv->busy_bag, real, stream);
 		else
@@ -363,7 +447,10 @@ camel_data_cache_get(CamelDataCache *cdc, const gchar *path, const gchar *key, C
  * Since: 2.26
  **/
 gchar *
-camel_data_cache_get_filename (CamelDataCache *cdc, const gchar *path, const gchar *key, CamelException *ex)
+camel_data_cache_get_filename (CamelDataCache *cdc,
+                               const gchar *path,
+                               const gchar *key,
+                               CamelException *ex)
 {
 	gchar *real;
 
@@ -384,7 +471,10 @@ camel_data_cache_get_filename (CamelDataCache *cdc, const gchar *path, const gch
  * Returns:
  **/
 gint
-camel_data_cache_remove(CamelDataCache *cdc, const gchar *path, const gchar *key, CamelException *ex)
+camel_data_cache_remove (CamelDataCache *cdc,
+                         const gchar *path,
+                         const gchar *key,
+                         CamelException *ex)
 {
 	CamelStream *stream;
 	gchar *real;
@@ -394,14 +484,15 @@ camel_data_cache_remove(CamelDataCache *cdc, const gchar *path, const gchar *key
 	stream = camel_object_bag_get(cdc->priv->busy_bag, real);
 	if (stream) {
 		camel_object_bag_remove(cdc->priv->busy_bag, stream);
-		camel_object_unref(stream);
+		g_object_unref (stream);
 	}
 
 	/* maybe we were a mem stream */
 	if (g_unlink (real) == -1 && errno != ENOENT) {
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Could not remove cache entry: %s: %s"),
-				      real, g_strerror (errno));
+		camel_exception_setv (
+			ex, CAMEL_EXCEPTION_SYSTEM,
+			_("Could not remove cache entry: %s: %s"),
+			real, g_strerror (errno));
 		ret = -1;
 	} else {
 		ret = 0;
@@ -412,42 +503,3 @@ camel_data_cache_remove(CamelDataCache *cdc, const gchar *path, const gchar *key
 	return ret;
 }
 
-/**
- * camel_data_cache_rename:
- * @cache:
- * @old:
- * @new:
- * @ex:
- *
- * Rename a cache path.  All cache items accessed from the old path
- * are accessible using the new path.
- *
- * CURRENTLY UNIMPLEMENTED
- *
- * Returns: -1 on error.
- **/
-gint camel_data_cache_rename(CamelDataCache *cache,
-			    const gchar *old, const gchar *new, CamelException *ex)
-{
-	/* blah dont care yet */
-	return -1;
-}
-
-/**
- * camel_data_cache_clear:
- * @cache:
- * @path: Path to clear, or NULL to clear all items in all paths.
- * @ex:
- *
- * Clear all items in a given cache path or all items in the cache.
- *
- * CURRENTLY_UNIMPLEMENTED
- *
- * Returns: -1 on error.
- **/
-gint
-camel_data_cache_clear(CamelDataCache *cache, const gchar *path, CamelException *ex)
-{
-	/* nor for this? */
-	return -1;
-}
