@@ -37,14 +37,45 @@
 
 G_DEFINE_TYPE (CamelTcpStream, camel_tcp_stream, CAMEL_TYPE_STREAM)
 
+struct _CamelTcpStreamPrivate {
+	gchar *socks_host;
+	gint socks_port;
+};
+
+#define CAMEL_TCP_STREAM_GET_PRIVATE(obj) \
+	(G_TYPE_INSTANCE_GET_PRIVATE \
+	((obj), CAMEL_TYPE_TCP_STREAM, CamelTcpStreamPrivate))
+
+static void
+camel_tcp_stream_finalize (GObject *object)
+{
+	CamelTcpStream *stream = CAMEL_TCP_STREAM (object);
+	CamelTcpStreamPrivate *priv = stream->priv;
+
+	priv = stream->priv;
+	g_free (priv->socks_host);
+	priv->socks_host = NULL;
+
+	G_OBJECT_CLASS (camel_tcp_stream_parent_class)->finalize (object);
+}
+
 static void
 camel_tcp_stream_class_init (CamelTcpStreamClass *class)
 {
+	GObjectClass *object_class;
+
+	g_type_class_add_private (class, sizeof (CamelTcpStreamPrivate));
+
+	object_class = G_OBJECT_CLASS (class);
+	object_class->finalize = camel_tcp_stream_finalize;
 }
 
 static void
 camel_tcp_stream_init (CamelTcpStream *tcp_stream)
 {
+	tcp_stream->priv = CAMEL_TCP_STREAM_GET_PRIVATE (tcp_stream);
+	tcp_stream->priv->socks_host = NULL;
+	tcp_stream->priv->socks_port = 0;
 }
 
 /**
@@ -170,3 +201,61 @@ camel_tcp_stream_get_remote_address (CamelTcpStream *stream,
 
 	return class->get_remote_address (stream, len);
 }
+
+/**
+ * camel_tcp_stream_set_socks_proxy:
+ * @stream: a #CamelTcpStream object
+ * @socks_host: hostname to use for the SOCKS proxy
+ * @socks_port: port number to use for the SOCKS proxy
+ *
+ * Configures a SOCKS proxy for the specified @stream.  Instead of direct connections,
+ * this @stream will instead go through the proxy.
+ *
+ * Since: 3.0
+ */
+void
+camel_tcp_stream_set_socks_proxy (CamelTcpStream *stream, const gchar *socks_host, gint socks_port)
+{
+	CamelTcpStreamPrivate *priv;
+
+	g_return_if_fail (CAMEL_IS_TCP_STREAM (stream));
+
+	priv = stream->priv;
+
+	g_free (priv->socks_host);
+
+	if (socks_host && socks_host[0] != '\0') {
+		priv->socks_host = g_strdup (socks_host);
+		priv->socks_port = socks_port;
+	} else {
+		priv->socks_host = NULL;
+		priv->socks_port = 0;
+	}
+}
+
+/**
+ * camel_tcp_stream_peek_socks_proxy:
+ * @stream: a #CamelTcpStream
+ * @socks_host_ret: location to return the name of the SOCKS host
+ * @socks_port_ret: location to return the port number in the SOCKS host
+ *
+ * Queries the SOCKS proxy that is configured for a @stream.  This will
+ * return #NULL in @socks_host_ret if no proxy is configured.
+ *
+ * Since: 3.0
+ */
+void
+camel_tcp_stream_peek_socks_proxy (CamelTcpStream *stream, const gchar **socks_host_ret, gint *socks_port_ret)
+{
+	CamelTcpStreamPrivate *priv;
+
+	g_return_if_fail (CAMEL_IS_TCP_STREAM (stream));
+
+	priv = stream->priv;
+
+	if (socks_host_ret)
+		*socks_host_ret = priv->socks_host;
+
+	if (socks_port_ret)
+		*socks_port_ret = priv->socks_port;
+}
