@@ -34,14 +34,13 @@
 #include <ssl.h>
 #endif /* HAVE_NSS */
 
-#include <glib.h>
 #include <glib/gi18n-lib.h>
 
 #include "camel.h"
 #include "camel-certdb.h"
 #include "camel-debug.h"
 #include "camel-provider.h"
-#include "camel-private.h"
+#include "camel-win32.h"
 
 #ifdef HAVE_NSS
 /* To protect NSS initialization and shutdown. This prevents
@@ -64,15 +63,15 @@ gint camel_application_is_exiting = FALSE;
 static gint
 nss_has_system_db(void)
 {
-	gint found = FALSE;
+	int found = FALSE;
 #ifndef G_OS_WIN32
 	FILE *f;
-	gchar buf[80];
+	char buf[80];
 
 	f = fopen(NSS_SYSTEM_DB "/pkcs11.txt", "r");
 	if (!f)
 		return FALSE;
-
+		
 	/* Check whether the system NSS db is actually enabled */
 	while (fgets(buf, 80, f) && !found) {
 		if (!strcmp(buf, "library=libnsssysinit.so\n"))
@@ -97,9 +96,6 @@ camel_init (const gchar *configdir, gboolean nss_init)
 
 	camel_debug_init();
 
-	/* initialise global camel_object_type */
-	camel_object_get_type();
-
 #ifdef HAVE_NSS
 	if (nss_init) {
 		gchar *nss_configdir = NULL;
@@ -145,6 +141,7 @@ camel_init (const gchar *configdir, gboolean nss_init)
 #endif
 		}
 
+
 #if NSS_VMAJOR > 3 || (NSS_VMAJOR == 3 && NSS_VMINOR >= 12)
 		/* See: https://wiki.mozilla.org/NSS_Shared_DB,
 		 * particularly "Mode 3A".  Note that the target
@@ -216,7 +213,7 @@ skip_nss_init:
 	/* set this certdb as the default db */
 	camel_certdb_set_default (certdb);
 
-	camel_object_unref (certdb);
+	g_object_unref (certdb);
 
 	initialised = TRUE;
 
