@@ -20,19 +20,23 @@ typedef enum _camel_imapx_id_t {
 	IMAPX_BODYSTRUCTURE,
 	IMAPX_BYE,
 	IMAPX_CAPABILITY,
+	IMAPX_CLOSED,
 	IMAPX_COPYUID,
 	IMAPX_ENVELOPE,
 	IMAPX_EXISTS,
 	IMAPX_EXPUNGE,
 	IMAPX_FETCH,
 	IMAPX_FLAGS,
+	IMAPX_HIGHESTMODSEQ,
 	IMAPX_INTERNALDATE,
 	IMAPX_LIST,
 	IMAPX_LSUB,
 	IMAPX_MESSAGES,
+	IMAPX_MODSEQ,
 	IMAPX_NAMESPACE,
 	IMAPX_NEWNAME,
 	IMAPX_NO,
+	IMAPX_NOMODSEQ,
 	IMAPX_OK,
 	IMAPX_PARSE,
 	IMAPX_PERMANENTFLAGS,
@@ -49,6 +53,7 @@ typedef enum _camel_imapx_id_t {
 	IMAPX_UIDVALIDITY,
 	IMAPX_UNSEEN,
 	IMAPX_UIDNEXT,
+	IMAPX_VANISHED,
 } camel_imapx_id_t;
 
 /* str MUST be in upper case, tokenised using gperf function */
@@ -61,6 +66,7 @@ enum {
 
 /* ********************************************************************** */
 
+GPtrArray *imapx_parse_uids (CamelIMAPXStream *is, CamelException *ex);
 void imapx_parse_flags(struct _CamelIMAPXStream *stream, guint32 *flagsp, struct _CamelFlag **user_flagsp, CamelException *ex);
 void imapx_write_flags(CamelStream *stream, guint32 flags, struct _CamelFlag *user_flags, CamelException *ex);
 gboolean imapx_update_message_info_flags (CamelMessageInfo *info, guint32 server_flags, CamelFlag *server_user_flags, CamelFolder *folder);
@@ -77,8 +83,12 @@ enum {
 	IMAPX_CAPABILITY_NAMESPACE		= (1 << 3),
 	IMAPX_CAPABILITY_UIDPLUS		= (1 << 4),
 	IMAPX_CAPABILITY_LITERALPLUS		= (1 << 5),
-	IMAPX_CAPABILITY_STARTTLS                = (1 << 6),
+	IMAPX_CAPABILITY_STARTTLS		= (1 << 6),
 	IMAPX_CAPABILITY_IDLE			= (1 << 7),
+	IMAPX_CAPABILITY_CONDSTORE		= (1 << 8),
+	IMAPX_CAPABILITY_QRESYNC		= (1 << 9),
+	IMAPX_CAPABILITY_LIST_STATUS		= (1 << 10),
+	IMAPX_CAPABILITY_LIST_EXTENDED		= (1 << 11),
 };
 
 struct _capability_info {
@@ -111,6 +121,7 @@ struct _fetch_info {
 	guint32 size;		/* RFC822.SIZE */
 	guint32 offset;		/* start offset of a BODY[]<offset.length> request */
 	guint32 flags;		/* FLAGS */
+	guint64 modseq;		/* MODSEQ */
 	struct _CamelFlag *user_flags;
 	gchar *date;		/* INTERNALDATE */
 	gchar *section;		/* section for a BODY[section] request */
@@ -128,6 +139,7 @@ struct _fetch_info {
 #define FETCH_DATE (1<<8)
 #define FETCH_SECTION (1<<9)
 #define FETCH_UID (1<<10)
+#define FETCH_MODSEQ (1<<11)
 
 struct _fetch_info *imapx_parse_fetch(struct _CamelIMAPXStream *is, CamelException *ex);
 void imapx_free_fetch(struct _fetch_info *finfo);
@@ -137,7 +149,7 @@ void imapx_dump_fetch(struct _fetch_info *finfo);
 
 struct _status_info {
 	camel_imapx_id_t result; /* ok/no/bad/preauth only, user_cancel - client response */
-	camel_imapx_id_t condition; /* read-only/read-write/alert/parse/trycreate/newname/permanentflags/uidvalidity/unseen */
+	camel_imapx_id_t condition; /* read-only/read-write/alert/parse/trycreate/newname/permanentflags/uidvalidity/unseen/highestmodseq */
 
 	union {
 		struct {
@@ -145,18 +157,20 @@ struct _status_info {
 			gchar *newname;
 		} newname;
 		guint32 permanentflags;
-		guint32 uidvalidity;
+		guint64 uidvalidity;
 		guint32 uidnext;
 		guint32 unseen;
+		guint64 highestmodseq;
 		struct {
-			guint32 uidvalidity;
+			guint64 uidvalidity;
 			guint32 uid;
 		} appenduid;
 		struct {
-			guint32 uidvalidity;
+			guint64 uidvalidity;
 			GPtrArray *uids;
 			GPtrArray *copied_uids;
 		} copyuid;
+		struct _capability_info *cinfo;
 	} u;
 
 	gchar *text;
@@ -169,11 +183,13 @@ void imapx_free_status(struct _status_info *sinfo);
 /* ********************************************************************** */
 /* parses the response from the status command */
 struct _state_info {
+	gchar *name;
 	guint32 messages;
 	guint32 recent;
 	guint32 uidnext;
-	guint32 uidvalidity;
 	guint32 unseen;
+	guint64 uidvalidity;
+	guint64 highestmodseq;
 };
 
 /* use g_free to free the return value */
@@ -214,6 +230,19 @@ guchar imapx_is_mask(const gchar *p);
 
 #define imapx_is_atom(s) (imapx_is_mask(s) & IMAPX_TYPE_ATOM_CHAR)
 
+extern gint camel_imapx_debug_flags;
+#define CAMEL_IMAPX_DEBUG_command	(1<<0)
+#define CAMEL_IMAPX_DEBUG_debug		(1<<1)
+#define CAMEL_IMAPX_DEBUG_extra		(1<<2)
+#define CAMEL_IMAPX_DEBUG_io		(1<<3)
+#define CAMEL_IMAPX_DEBUG_token		(1<<4)
+#define CAMEL_IMAPX_DEBUG_parse		(1<<5)
+
+/* Set this to zero to remove all debug output at build time */
+#define CAMEL_IMAPX_DEBUG_ALL		((1<<6)-1)
+
+#define camel_imapx_debug(type, ...) do { if (camel_imapx_debug_flags & CAMEL_IMAPX_DEBUG_ALL & CAMEL_IMAPX_DEBUG_ ## type) { __VA_ARGS__ ; } } while (0)
+
 /* ********************************************************************** */
 
 void imapx_utils_init(void);
