@@ -419,11 +419,8 @@ static gboolean
 nntp_disconnect_online (CamelService *service, gboolean clean, CamelException *ex)
 {
 	CamelNNTPStore *store = CAMEL_NNTP_STORE (service);
-	CamelServiceClass *service_class;
 	gchar *line;
 
-	service_class = CAMEL_SERVICE_GET_CLASS (service);
-
 	camel_service_lock (CAMEL_SERVICE (store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 
 	if (clean) {
@@ -431,11 +428,6 @@ nntp_disconnect_online (CamelService *service, gboolean clean, CamelException *e
 		camel_exception_clear(ex);
 	}
 
-	if (!service_class->disconnect (service, clean, ex)) {
-		camel_service_unlock (CAMEL_SERVICE (store), CAMEL_SERVICE_REC_CONNECT_LOCK);
-		return FALSE;
-	}
-
 	g_object_unref (store->stream);
 	store->stream = NULL;
 	g_free(store->current_folder);
@@ -450,12 +442,6 @@ static gboolean
 nntp_disconnect_offline (CamelService *service, gboolean clean, CamelException *ex)
 {
 	CamelDiscoStore *disco = CAMEL_DISCO_STORE(service);
-	CamelServiceClass *service_class;
-
-	/* Chain up to parent's disconnect() method. */
-	service_class = CAMEL_SERVICE_CLASS (camel_nntp_store_parent_class);
-	if (!service_class->disconnect (service, clean, ex))
-		return FALSE;
 
 	if (disco->diary) {
 		g_object_unref (disco->diary);
@@ -707,6 +693,33 @@ nntp_store_get_subscribed_folder_info (CamelNNTPStore *store, const gchar *top,
 	return first;
 }
 
+static CamelFolderInfo *
+tree_insert(CamelFolderInfo *root, CamelFolderInfo *last, CamelFolderInfo *fi)
+{
+	CamelFolderInfo *kfi;
+
+	if (!root)
+		root = fi;
+	else if (!last) {
+		kfi = root;
+		while (kfi->next)
+			kfi = kfi->next;
+		kfi->next = fi;
+		fi->parent = kfi->parent;
+	} else {
+		if (!last->child) {
+			last->child = fi;
+			fi->parent = last;
+		} else {
+			kfi = last->child;
+			while (kfi->next)
+				kfi = kfi->next;
+			kfi->next = fi;
+			fi->parent = last;
+		}
+	}
+	return root;
+}
 /* returns new root */
 static CamelFolderInfo *
 nntp_push_to_hierarchy (CamelURL *base_url, CamelFolderInfo *root, CamelFolderInfo *pfi, GHashTable *known)
@@ -742,21 +755,8 @@ nntp_push_to_hierarchy (CamelURL *base_url, CamelFolderInfo *root, CamelFolderIn
 			camel_url_free (url);
 
 			g_hash_table_insert (known, fi->full_name, fi);
-			if (last) {
-				if (!last->child) {
-					last->child = fi;
-					fi->parent = last;
-				} else {
-					kfi = last->child;
-					while (kfi->next)
-						kfi = kfi->next;
-					kfi->next = fi;
-					fi->parent = last;
-				}
-			}
+			root = tree_insert(root, last, fi);
 			last = fi;
-			if (!root)
-				root = fi;
 		} else {
 			last = kfi;
 		}
@@ -768,28 +768,7 @@ nntp_push_to_hierarchy (CamelURL *base_url, CamelFolderInfo *root, CamelFolderIn
 	g_free (pfi->name);
 	pfi->name = g_strdup (name);
 
-	if (!root) {
-		root = pfi;
-	} else if (!last) {
-		kfi = root;
-		while (kfi->next)
-			kfi = kfi->next;
-		kfi->next = pfi;
-		pfi->parent = kfi->parent;
-	} else {
-		if (!last->child) {
-			last->child = pfi;
-			pfi->parent = last;
-		} else {
-			kfi = last->child;
-			while (kfi->next)
-				kfi = kfi->next;
-			kfi->next = pfi;
-			pfi->parent = last;
-		}
-	}
-
-	return root;
+	return tree_insert(root, last, pfi);
 }
 
 /*
