@@ -5,13 +5,9 @@
 #include "folders.h"
 #include "session.h"
 
-#include <camel/camel-exception.h>
-#include <camel/camel-service.h>
-#include <camel/camel-store.h>
-
 static const gchar *nntp_drivers[] = { "nntp" };
 
-static gchar *remote_providers[] = {
+static const gchar *remote_providers[] = {
 	"NNTP_TEST_URL",
 };
 
@@ -48,7 +44,7 @@ gint main(gint argc, gchar **argv)
 		camel_test_fatal();
 	}
 
-	camel_object_unref((CamelObject *)session);
+	g_object_unref (session);
 	camel_exception_free(ex);
 
 	return 0;
