@@ -29,6 +29,10 @@
 #include <glib/gi18n-lib.h>
 #include "e-name-selector-model.h"
 
+#define E_NAME_SELECTOR_MODEL_GET_PRIVATE(obj) \
+	(G_TYPE_INSTANCE_GET_PRIVATE \
+	((obj), E_TYPE_NAME_SELECTOR_MODEL, ENameSelectorModelPrivate))
+
 typedef struct {
 	gchar              *name;
 	gchar              *pretty_name;
@@ -37,6 +41,13 @@ typedef struct {
 }
 Section;
 
+struct _ENameSelectorModelPrivate {
+	GArray *sections;
+	EContactStore *contact_store;
+	ETreeModelGenerator *contact_filter;
+	GHashTable *destination_uid_hash;
+};
+
 static gint generate_contact_rows  (EContactStore *contact_store, GtkTreeIter *iter,
 				    ENameSelectorModel *name_selector_model);
 static void override_email_address (EContactStore *contact_store, GtkTreeIter *iter,
@@ -63,66 +74,75 @@ G_DEFINE_TYPE (ENameSelectorModel, e_name_selector_model, G_TYPE_OBJECT)
 static void
 e_name_selector_model_init (ENameSelectorModel *name_selector_model)
 {
-	name_selector_model->sections       = g_array_new (FALSE, FALSE, sizeof (Section));
-	name_selector_model->contact_store  = e_contact_store_new ();
+	name_selector_model->priv =
+		E_NAME_SELECTOR_MODEL_GET_PRIVATE (name_selector_model);
+	name_selector_model->priv->sections       = g_array_new (FALSE, FALSE, sizeof (Section));
+	name_selector_model->priv->contact_store  = e_contact_store_new ();
 
-	name_selector_model->contact_filter =
-		e_tree_model_generator_new (GTK_TREE_MODEL (name_selector_model->contact_store));
-	e_tree_model_generator_set_generate_func (name_selector_model->contact_filter,
+	name_selector_model->priv->contact_filter =
+		e_tree_model_generator_new (GTK_TREE_MODEL (name_selector_model->priv->contact_store));
+	e_tree_model_generator_set_generate_func (name_selector_model->priv->contact_filter,
 						  (ETreeModelGeneratorGenerateFunc) generate_contact_rows,
 						  name_selector_model, NULL);
-	e_tree_model_generator_set_modify_func (name_selector_model->contact_filter,
+	e_tree_model_generator_set_modify_func (name_selector_model->priv->contact_filter,
 						(ETreeModelGeneratorModifyFunc) override_email_address,
 						name_selector_model, NULL);
 
-	g_object_unref (name_selector_model->contact_store);
+	g_object_unref (name_selector_model->priv->contact_store);
 
-	name_selector_model->destination_uid_hash = NULL;
+	name_selector_model->priv->destination_uid_hash = NULL;
 }
 
 static void
-e_name_selector_model_finalize (GObject *object)
+name_selector_model_finalize (GObject *object)
 {
-	ENameSelectorModel *name_selector_model = E_NAME_SELECTOR_MODEL (object);
-	gint                i;
+	ENameSelectorModelPrivate *priv;
+	gint i;
 
-	for (i = 0; i < name_selector_model->sections->len; i++)
-		free_section (name_selector_model, i);
+	priv = E_NAME_SELECTOR_MODEL_GET_PRIVATE (object);
 
-	g_array_free (name_selector_model->sections, TRUE);
-	g_object_unref (name_selector_model->contact_filter);
+	for (i = 0; i < priv->sections->len; i++)
+		free_section (E_NAME_SELECTOR_MODEL (object), i);
 
-	if (name_selector_model->destination_uid_hash)
-		g_hash_table_destroy (name_selector_model->destination_uid_hash);
+	g_array_free (priv->sections, TRUE);
+	g_object_unref (priv->contact_filter);
 
-	if (G_OBJECT_CLASS (e_name_selector_model_parent_class)->finalize)
-		G_OBJECT_CLASS (e_name_selector_model_parent_class)->finalize (object);
+	if (priv->destination_uid_hash)
+		g_hash_table_destroy (priv->destination_uid_hash);
+
+	/* Chain up to parent's finalize() method. */
+	G_OBJECT_CLASS (e_name_selector_model_parent_class)->finalize (object);
 }
 
 static void
-e_name_selector_model_class_init (ENameSelectorModelClass *name_selector_model_class)
+e_name_selector_model_class_init (ENameSelectorModelClass *class)
 {
-	GObjectClass *object_class = G_OBJECT_CLASS (name_selector_model_class);
+	GObjectClass *object_class;
 
-	object_class->finalize = e_name_selector_model_finalize;
+	g_type_class_add_private (class, sizeof (ENameSelectorModelPrivate));
 
-	signals [SECTION_ADDED] =
-		g_signal_new ("section-added",
-			      G_OBJECT_CLASS_TYPE (object_class),
-			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (ENameSelectorModelClass, section_added),
-			      NULL, NULL,
-			      g_cclosure_marshal_VOID__STRING,
-			      G_TYPE_NONE, 1, G_TYPE_STRING);
+	object_class = G_OBJECT_CLASS (class);
+	object_class->finalize = name_selector_model_finalize;
 
-	signals [SECTION_REMOVED] =
-		g_signal_new ("section-removed",
-			      G_OBJECT_CLASS_TYPE (object_class),
-			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (ENameSelectorModelClass, section_removed),
-			      NULL, NULL,
-			      g_cclosure_marshal_VOID__STRING,
-			      G_TYPE_NONE, 1, G_TYPE_STRING);
+	signals[SECTION_ADDED] = g_signal_new (
+		"section-added",
+		G_OBJECT_CLASS_TYPE (object_class),
+		G_SIGNAL_RUN_LAST,
+		G_STRUCT_OFFSET (ENameSelectorModelClass, section_added),
+		NULL, NULL,
+		g_cclosure_marshal_VOID__STRING,
+		G_TYPE_NONE, 1,
+		G_TYPE_STRING);
+
+	signals[SECTION_REMOVED] = g_signal_new (
+		"section-removed",
+		G_OBJECT_CLASS_TYPE (object_class),
+		G_SIGNAL_RUN_LAST,
+		G_STRUCT_OFFSET (ENameSelectorModelClass, section_removed),
+		NULL, NULL,
+		g_cclosure_marshal_VOID__STRING,
+		G_TYPE_NONE, 1,
+		G_TYPE_STRING);
 }
 
 /**
@@ -169,12 +189,12 @@ generate_contact_rows (EContactStore *contact_store, GtkTreeIter *iter,
 	if (!contact_uid)
 		return 0;  /* Can happen with broken databases */
 
-	for (i = 0; i < name_selector_model->sections->len; i++) {
+	for (i = 0; i < name_selector_model->priv->sections->len; i++) {
 		Section *section;
 		GList   *destinations;
 		GList   *l;
 
-		section = &g_array_index (name_selector_model->sections, Section, i);
+		section = &g_array_index (name_selector_model->priv->sections, Section, i);
 		destinations = e_destination_store_list_destinations (section->destination_store);
 
 		for (l = destinations; l; l = g_list_next (l)) {
@@ -240,7 +260,7 @@ HashCompare;
 static void
 emit_destination_uid_changes_cb (gchar *uid_num, gpointer value, HashCompare *hash_compare)
 {
-	EContactStore *contact_store = hash_compare->name_selector_model->contact_store;
+	EContactStore *contact_store = hash_compare->name_selector_model->priv->contact_store;
 
 	if (!hash_compare->other_hash || !g_hash_table_lookup (hash_compare->other_hash, uid_num)) {
 		GtkTreeIter  iter;
@@ -273,8 +293,8 @@ destinations_changed (ENameSelectorModel *name_selector_model)
 
 	destination_uid_hash_new = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 
-	for (i = 0; i < name_selector_model->sections->len; i++) {
-		Section *section = &g_array_index (name_selector_model->sections, Section, i);
+	for (i = 0; i < name_selector_model->priv->sections->len; i++) {
+		Section *section = &g_array_index (name_selector_model->priv->sections, Section, i);
 		GList   *destinations;
 		GList   *l;
 
@@ -294,8 +314,8 @@ destinations_changed (ENameSelectorModel *name_selector_model)
 		g_list_free (destinations);
 	}
 
-	destination_uid_hash_old = name_selector_model->destination_uid_hash;
-	name_selector_model->destination_uid_hash = destination_uid_hash_new;
+	destination_uid_hash_old = name_selector_model->priv->destination_uid_hash;
+	name_selector_model->priv->destination_uid_hash = destination_uid_hash_new;
 
 	hash_compare.name_selector_model = name_selector_model;
 
@@ -318,9 +338,9 @@ free_section (ENameSelectorModel *name_selector_model, gint n)
 	Section *section;
 
 	g_assert (n >= 0);
-	g_assert (n < name_selector_model->sections->len);
+	g_assert (n < name_selector_model->priv->sections->len);
 
-	section = &g_array_index (name_selector_model->sections, Section, n);
+	section = &g_array_index (name_selector_model->priv->sections, Section, n);
 
 	g_signal_handlers_disconnect_matched (section->destination_store, G_SIGNAL_MATCH_DATA,
 					      0, 0, NULL, NULL, name_selector_model);
@@ -337,8 +357,8 @@ find_section_by_name (ENameSelectorModel *name_selector_model, const gchar *name
 
 	g_assert (name != NULL);
 
-	for (i = 0; i < name_selector_model->sections->len; i++) {
-		Section *section = &g_array_index (name_selector_model->sections, Section, i);
+	for (i = 0; i < name_selector_model->priv->sections->len; i++) {
+		Section *section = &g_array_index (name_selector_model->priv->sections, Section, i);
 
 		if (!strcmp (name, section->name))
 			return i;
@@ -364,7 +384,7 @@ e_name_selector_model_peek_contact_store (ENameSelectorModel *name_selector_mode
 {
 	g_return_val_if_fail (E_IS_NAME_SELECTOR_MODEL (name_selector_model), NULL);
 
-	return name_selector_model->contact_store;
+	return name_selector_model->priv->contact_store;
 }
 
 /**
@@ -381,7 +401,7 @@ e_name_selector_model_peek_contact_filter (ENameSelectorModel *name_selector_mod
 {
 	g_return_val_if_fail (E_IS_NAME_SELECTOR_MODEL (name_selector_model), NULL);
 
-	return name_selector_model->contact_filter;
+	return name_selector_model->priv->contact_filter;
 }
 
 /**
@@ -402,8 +422,8 @@ e_name_selector_model_list_sections (ENameSelectorModel *name_selector_model)
 	g_return_val_if_fail (E_IS_NAME_SELECTOR_MODEL (name_selector_model), NULL);
 
 	/* Do this backwards so we can use g_list_prepend () and get correct order */
-	for (i = name_selector_model->sections->len - 1; i >= 0; i--) {
-		Section *section = &g_array_index (name_selector_model->sections, Section, i);
+	for (i = name_selector_model->priv->sections->len - 1; i >= 0; i--) {
+		Section *section = &g_array_index (name_selector_model->priv->sections, Section, i);
 		gchar   *name;
 
 		name = g_strdup (section->name);
@@ -456,7 +476,7 @@ e_name_selector_model_add_section (ENameSelectorModel *name_selector_model,
 	g_signal_connect_swapped (section.destination_store, "row-inserted",
 				  G_CALLBACK (destinations_changed), name_selector_model);
 
-	g_array_append_val (name_selector_model->sections, section);
+	g_array_append_val (name_selector_model->priv->sections, section);
 
 	destinations_changed (name_selector_model);
 	g_signal_emit (name_selector_model, signals [SECTION_ADDED], 0, name);
@@ -484,7 +504,7 @@ e_name_selector_model_remove_section (ENameSelectorModel *name_selector_model, c
 	}
 
 	free_section (name_selector_model, n);
-	g_array_remove_index_fast (name_selector_model->sections, n);  /* Order doesn't matter */
+	g_array_remove_index_fast (name_selector_model->priv->sections, n);  /* Order doesn't matter */
 
 	destinations_changed (name_selector_model);
 	g_signal_emit (name_selector_model, signals [SECTION_REMOVED], 0, name);
@@ -517,7 +537,7 @@ e_name_selector_model_peek_section (ENameSelectorModel *name_selector_model, con
 		return FALSE;
 	}
 
-	section = &g_array_index (name_selector_model->sections, Section, n);
+	section = &g_array_index (name_selector_model->priv->sections, Section, n);
 
 	if (pretty_name)
 		*pretty_name = g_strdup (section->pretty_name);
@@ -559,12 +579,12 @@ e_name_selector_model_get_contact_emails_without_used (ENameSelectorModel *name_
 	email_list = e_contact_get (contact, E_CONTACT_EMAIL);
 	emails = g_list_length (email_list);
 
-	for (i = 0; i < name_selector_model->sections->len; i++) {
+	for (i = 0; i < name_selector_model->priv->sections->len; i++) {
 		Section *section;
 		GList   *destinations;
 		GList   *l;
 
-		section = &g_array_index (name_selector_model->sections, Section, i);
+		section = &g_array_index (name_selector_model->priv->sections, Section, i);
 		destinations = e_destination_store_list_destinations (section->destination_store);
 
 		for (l = destinations; l; l = g_list_next (l)) {
